[KSplash Theme: Classic]
Name = Classic Splash Screen
Description = Splash Screen from the good old KDE SC 4 series
Version = 1.0
Author = Martin Klapetek <mklapetek(at)kde.org>
Homepage = http://www.kde.org
